;**********************************************************************************
;* R O U T I N E N   F U E R   D I S P L A Y   S A M S U N G   0 2 8 2 A          *
;* 2 4   Z E I C H E N   X   8   Z E I L E N                                      *
;*                                                                                *
;* Dateiname		:"0282Adef.inc"                                           *
;* Titel		:Registerdefinitionen und Routinen Ansteuerung eines      *
;*			 Displays Samsung 0282A durch Atmel-Controller            *
;* Datum 		:25.11.2006                                               *
;* getestet mit		:AT90S2313                                                *
;*                                                                                *
;* BESCHREIBUNG                                                                   *
;*                                                                                *
;* Es werden Routinen zum Initialisieren, Loeschen und zur Ausgabe von Zeichen    *
;* bzw. Strings fuer das Display Samsung 0282A mit 8 Zeilen zu 24 Zeichen         *
;* auf der Basis des IC's M50530 von Mitsubishi zur Verfuegung gestellt.          *
;* Die Ansteuerung basiert auf einer Datenbreite von 8 Bit.                       *
;*                                                                                *
;*                                                                                *
;**********************************************************************************

;Registerdefinitionen

	.def Zeile 	= r00	;0...8
	.def Spalte	= r01	;0...23
	.def Adresse	= r02	;Bit 0...4 - Spaltenadresse (0...23)
				;Bit 5...7 - Zeilenadresse (0...7)
	.def Zeichen	= r03	;ASCII-Zeichen

	.def TempA	= r16	;Hilfsregister
	.def TempB	= r17	;Hilfsregister
	.def TempC	= r18	;Hilfsregister

	.def RZlow	= r30	;Stringadresse low
	.def RZhigh	= r31	;Stringadresse high


;Portzuordnungen

	.equ	DATPORT	= PORTB	;Ausgaberegister Datenport
	.equ	DATPIN	= PINB	;Eingaberegister Datenport
	.equ	DATDDR	= DDRB	;Datenrichtungsregister Datenport
	.equ	STPORT	= PORTD	;Ausgaberegister Steuerport
	.equ	STPIN	= PIND	;Eingaberegister Steuerport
	.equ	STDDR	= DDRD	;Datenrichtungsregister Steuerport

;Bitzuordnung Steuerport
	.equ	EX	= 0	;Enable
	.equ	R_W	= 1	;R/W
	.equ	IOC1	= 2	;I/O-Select 1
	.equ	IOC2	= 3	;I/O-Select 2




;	----------------------------------------------------
;	***  Subroutinen zur Ansteuerung eines Displays  ***
;	***  Samsung 0282A (M50530) mit 8 x 24 Zeichen   ***
;	----------------------------------------------------

;	---  Ausgabe eines Strings auf Kursoradresse  ---
StrCur:	push	TempB		;Register retten
	rjmp	Str02

;	---  Ausgabe eines Strings auf Zeile/Spalte  ---
;	---  Ausgabezeile im Register "Zeile"         ---
;	---  Ausgabespalte im Register "Spalte"       ---
StrZeSp:push	TempB		;Register retten
	rcall	ChZeSp		;Adresse wandeln
	rjmp	Str01

;	---  Ausgabe eines Strings auf Adresse     ---
;	---  Ausgabeadresse im Register "Adresse"  ---
StrAdr:	push	TempB		;Register retten
	rcall	ChAdr		;Adresse wandeln
Str01:	rcall	CursOut		;Kursoradresse setzen
Str02:	push	R0		;Register 0 retten
Str03:	lpm			;1 Byte aus Programmspeicher holen
	adiw	RZlow,1		;Stringadresse erhoeen
	tst	R0		;Test auf 0
	breq	Str04		;ja, Sprung
	mov	TempB,R0	;Ausgabe vorbereiten
	rcall	ChrOut		;Zeichen auf Display ausgeben
	rjmp	Str03		;Sprung zum Holen naehstes Byte
Str04:	pop	R0		;Register 0 wiederherstellen
	pop	TempB		;Register wiederherstellen
	ret

;	---  Ausgabe eines Zeichens auf aktuelle Kursoradresse  ---
ChrCur:	push	TempB		;Register retten
	rjmp	Chr02

;	---  Ausgabe eines Zeichens auf Zeile/Spalte  ---
;	---  Ausgabezeile im Register "Zeile"         ---
;	---  Ausgabespalte im Register "Spalte"       ---
ChrZeSp:push	TempB		;Register retten
	rcall	ChZeSp		;Adresse wandeln
	rjmp	Chr01

;	---  Ausgabe eines Zeichens auf aktuelle Kursoradresse  ---
;	---  Ausgabeadresse im Register "Adresse"  ---
ChrAdr:	push	TempB		;Register retten
	rcall	ChAdr		;Adresse wandeln
Chr01:	rcall	CursOut		;Kursoradresse setzen
Chr02:	mov	TempB,Zeichen	;Zeichen holen
	rcall	ChrOut		;Zeichen auf Kursoradresse ausgeben
	pop	TempB		;Register wiederherstellen
	ret

;	---  Adressinformation aus Zeile/Spalte   ---
;	---  in physische Zeichenadresse wandeln  ---
;	---  Ausgabe in Register TempB            ---
ChZeSp:	push	TempA		;Register retten
	push	TempC		;Register retten
	mov	TempA,Zeile	;Zeile holen
	mov	TempC,Spalte	;Spalet holen
	rjmp	Ch01
;	---  Adressinformation aus Adresse        ---
;	---  in physische Zeichenadresse wandeln  ---
;	---  Ausgabe in Register TempB            ---
ChAdr:	push	TempA
	push	TempC		;Register retten
	ldi	TempA,0xE0	;Bits 5...7 maskieren
	and	TempA,Adresse	;Zeilenadresse in TempA
	ldi	TempC,0x1F	;Bits 0...4 maskieren
	and	TempC,Adresse	;Spaltenadresse in TempC
	swap	TempA		;
	lsr	TempA		;Bits 5...7 nach 
Ch01:	lsl	TempA
	sbrc	TempA,0x03
	sbr	TempA,0x01
	cbr	TempA,0x08
	inc	TempA
	push	TempC
	ldi	TempB,0x00
	ldi	TempC,0x18
Ch02:	dec	TempA
	breq	Ch03
	add	TempB,TempC
	rjmp	Ch02
Ch03:	pop	TempC
	add	TempB,TempC
	pop	TempC		;Register wiederherstellen
	pop	TempA		;Register wiederherstellen
	ret

;	---  Ausgabe eines Bytes  ---
;	Byte in "TempB"

SteuOut:push	TempA		;Schreiben eines Steuerzeichens
	ldi	TempA,0x00
	rjmp	Out
ChrOut:	push	TempA		;Schreiben eines ASCII-Zeichens
	ldi	TempA,0x04
	rjmp	Out
DspOut:	push	TempA		;Schreiben der Display-Startadresse
	ldi	TempA,0x08
	rjmp	Out
CursOut:push	TempA		;Schreiben der Kursoradresse
	ldi	TempA,0x0C

Out:	rcall	RBusy		;Warten auf BUSY-Flag
	out	DATPORT,TempB	;Schreiben des Daten-Bytes auf den Datenport
	push	TempB		;Register retten
	ser	TempB		;Datenport Bit 0..7 auf Output
	out	DATDDR,TempB	;Datenport Richtungsregister schreiben
	out	STPORT,TempA	;Schreiboperation vorbereiten
	nop
	sbi	STPORT,EX	;Aktivierungsbit setzen
	nop
	nop
	cbi	STPORT,EX	;Aktivierungsbit loeschen
	nop
	clr	TempA		;Register loeschen (0x00)
	out	STPORT,TempA	;Steuerport zurcksetzen
	out	DATDDR,TempA	;Datenport Bit 0..7 auf Input
				;Datenport Richtungsregister schreiben
	pop	TempB		;Register zurckholen
	pop	TempA		;Register zurckholen
	ret


;	---  Warten auf Busy-Flag  ---

RBusy:	push	TempA
	in	TempA,STDDR	;Steuerport Richtungsregister holen
	ori	TempA,0x0F	;Steuerport Bit 0..3 auf Output, Rest unveraendert
	out	STDDR,TempA	;Steuerport Richtungsregister schreiben
	clr	TempA		;Datenport Bit 0..7 auf Input
	out	DATDDR,TempA	;Datenport Richtungsregister schreiben
	ldi	TempA,0x02	;Flags lesen vorbereiten
	out	STPORT,TempA	;Steuerport setzen
RB1:	nop
	sbi	STPORT,EX	;Aktivierungsbit setzen
	nop
	nop
	nop
	in	TempA,DATPIN	;Statusflags lesen
	cbi	STPORT,EX	;Aktivierungsbit loeschen
	sbrc	TempA,7		;Busy-Flag geloescht?
	rjmp	RB1		;nein, Sprung zur naechsten Abfrage
	nop
	ldi	TempA,0x00	;Lesen deaktivieren
	out	STPORT,TempA	;Steuerport setzen
	pop	TempA		;Register rckretten
	ret

;	---  Display initialisieren / loeschen  ---

InitDsp:push	TempB
	ldi	TempB,0xFA	;SF, I/O=8, Font=8, DUTY=4x, RAM=192Byte
	rcall	SteuOut
	ldi	TempB,0x50	;SE, Cursor inc after writing Data (WD)
	rcall	SteuOut
	ldi	TempB,0x30	;SD, 
	rcall	SteuOut
;	ldi	TempB,0x0B	;SU
;	rcall	SteuOut
;	ldi	TempB,0x05	;SB
;	rcall	SteuOut
	pop	TempB
ClrDsp:	push	TempB
	ldi	TempB,0x01	;CH, Display loeschen
	rcall	SteuOut
	ldi	TempB,0x00	;Display-Startadresse
	rcall	DspOut
	ldi	TempB,0x00	;Kursor-Startadresse
	rcall	CursOut
	pop	TempB
	ret
